from typing import List

try:
    import ujson as json
except ImportError:
    import json


class MList(list):
    """列表类"""

    @classmethod
    def quick_value(cls, ls: List, substitute: str = "") -> str:
        """快速列表取值"""
        if len(ls) == 0:
            return substitute
        return ls[0].strip() if (ls[0] and ls[0].strip()) else substitute

    @classmethod
    def drop_empty_from_list(cls, lst: List) -> List[str]:
        """去除列表中的空字符串元素"""

        return [i for i in lst if i.strip()]

    @classmethod
    def split_and_drop_empty(cls, s: str, sep: str) -> List[str]:
        """切割字符串并去除空字符串元素"""
        return [i.strip() for i in s.split(sep) if i.strip()]

    @classmethod
    def json_list_stringify_limit(cls, lst: List, limit=1024) -> str:
        """列表转json列表长度自动限制(丢弃超出限制的元素)"""
        while True:
            if len(lst) < 1:
                return "[]"
            res = json.dumps(lst, ensure_ascii=False)
            if len(res) <= limit:
                return res
            lst = lst[:-1]



from typing import Dict


class MDict:
    @classmethod
    def merge_dicts(cls, *args: Dict) -> Dict:
        """合并多个字典的键值对 后面的字典会覆盖前面"""

        res = {}
        for d in args:
            for k, v in d.items():
                if d.get(k):
                    res[k] = v
        return res



from typing import Any, Optional


class MConvert:
    """转换类"""

    @classmethod
    def safe_int(cls, s: Any, substitute: Optional[int] = None) -> Optional[int]:
        """不会报错的int转换

        :param s: 要转换的任意类型
        """
        try:
            return int(s)
        except Exception:
            return substitute

    @classmethod
    def safe_float(cls, s: Any, substitute: Optional[float] = None) -> Optional[float]:
        """不会报错的float转换

        :param s: 要转换的任意类型
        """
        try:
            return float(s)
        except Exception:
            return substitute



from typing import Dict, Optional, Union

import aiohttp
import requests

try:
    import ujson as json
except ImportError:
    import json


class MFetch:
    """请求类"""

    @classmethod
    def fetch(
        cls,
        url,
        method: str = "get",
        params: Optional[Dict] = None,
        data: Union[str, Dict] = "{}",
        headers: Optional[Dict] = None,
        proxy_server: str = "",
        timeout: int = 60,
    ) -> str:
        """发起请求"""

        if headers is None:
            headers = {}
        if params is None:
            params = {}
        if isinstance(data, dict):
            data = json.dumps(data)

        if proxy_server:
            proxies = {
                "http": f"http://{proxy_server}",
                "https": f"https://{proxy_server}",
            }
        else:
            proxies = None

        resp = getattr(requests, method)(
            url,
            params=params,
            data=data,
            headers=headers,
            proxies=proxies,
            timeout=timeout,
        )
        return resp.text

    @classmethod
    async def async_fetch(
        cls,
        url,
        method: str = "get",
        params: Optional[Dict] = None,
        data: Union[str, Dict] = "{}",
        headers: Optional[Dict] = None,
        proxy_server: str = "",
        timeout: int = 60,
    ) -> str:
        """发起异步请求"""

        if headers is None:
            headers = {}
        if params is None:
            params = {}
        if isinstance(data, dict):
            data = json.dumps(data)

        async with aiohttp.ClientSession(headers=headers) as session:
            if proxy_server:
                conn = aiohttp.TCPConnector(limit=10, verify_ssl=False)
                session = aiohttp.ClientSession(connector=conn)
                session._default_headers.update(  # noqa: SLF001
                    {"Proxy-Switch-Ip": "yes"},
                )
                session._default_headers.update(  # noqa: SLF001
                    {"Proxy-Server": proxy_server},
                )
            async with getattr(session, method)(
                url,
                params=params,
                data=data,
                timeout=timeout,
            ) as resp:
                return await resp.text()
